/**
 * \file main.c
 * \brief Firmware for the USB-Servo
 * \author Ronald Schaten
 * \version $Id: main.c,v 1.1 2006/10/28 12:40:42 rschaten Exp $
 *
 * License: See documentation.
 */

#define F_CPU 12000000UL
 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <compat/twi.h>

#define MAX_TRIES 50

#define I2C_START 0
#define I2C_DATA 1
#define I2C_DATA_ACK 2
#define I2C_STOP 3
#define ACK 1
#define NACK 0

#include "usbdrv.h"
#include "oddebug.h"
#include "usbservo.h"

extern volatile unsigned char REG_ID,REG_VH,REG_VL;
extern volatile unsigned int VAL_ch1,VAL_ch2,VAL_ch3,VAL_ch4,VAL_ch5,VAL_ch6,VAL_ch7,VAL_ch8;
extern volatile unsigned int VAL_angle,VAL_gyro;
extern volatile unsigned int VAL_buttons;

volatile unsigned char REG_ID = 0;
volatile unsigned char REG_VH = 0;
volatile unsigned char REG_VL = 0;

volatile unsigned int VAL_angle = 0;
volatile unsigned int VAL_gyro = 0;

volatile unsigned int VAL_ch1 = 0;
volatile unsigned int VAL_ch2 = 0;
volatile unsigned int VAL_ch3 = 0;
volatile unsigned int VAL_ch4 = 0;
volatile unsigned int VAL_ch5 = 0;
volatile unsigned int VAL_ch6 = 0;
volatile unsigned int VAL_ch7 = 0;
volatile unsigned int VAL_ch8 = 0;

volatile unsigned int VAL_buttons = 0;

/** Global variable, contains the angle of the servo. */
static uint8_t angle;

/**
 * USB-Setup-Handler. Handles setup-calls that are received from the
 * USB-Interface.
 * \param data Eight bytes of data.
 * \return The number of returned bytes (in replyBuffer[]).
 */
uchar usbFunctionSetup(uchar data[8]) {
    static uchar replyBuffer[8];
    uchar replyLength;

    replyBuffer[0] = msgOK;
    switch (data[1]) {	
		case CMD_GET_CH1_4:             //===[ CH1~4 nm]=============== 

				//====[ch1]================
				replyBuffer[0] = VAL_ch1;
				replyBuffer[1] = VAL_ch1 >> 8;
		
				//====[ch2]=====================
				replyBuffer[2] = VAL_ch2;
				replyBuffer[3] = VAL_ch2 >> 8;		

				//====[ch3]================
				replyBuffer[4] = VAL_ch3;
				replyBuffer[5] = VAL_ch3 >> 8;
		
				//====[ch4]=====================
				replyBuffer[6] = VAL_ch4;
				replyBuffer[7] = VAL_ch4 >> 8;	
				
				replyLength = 8;
				
				break;	
				
		case CMD_GET_CH5_6_B:           //===[ CH5~6 nm,H Buttons]===============

				//====[ch5]================
				replyBuffer[0] = VAL_ch5;
				replyBuffer[1] = VAL_ch5 >> 8;
		
				//====[ch6]=====================
				replyBuffer[2] = VAL_ch6;
				replyBuffer[3] = VAL_ch6 >> 8;		

				//====[Buttons]================
				replyBuffer[4] = VAL_buttons;
				replyBuffer[5] = VAL_buttons >> 8;
		
				//====[ch4]=====================
				replyBuffer[6] = 0;
				replyBuffer[7] = 0;	
				
				replyLength = 8;
				
				break;	
						
		case CMD_ECHO:             /* echo */
				replyBuffer[0] = data[2];
				replyBuffer[1] = data[3];
				replyLength = 2;
				break;
		case CMD_GET:              /* read status */
				replyBuffer[0] = angle;
				replyLength = 1;
				break;
		case CMD_SET:              /* set status */
				angle = data[2];
				replyLength = 0;
				break;
		default:                   /* WTF? */
				replyBuffer[0] = msgErr;
				replyLength = 1;
				break;
    }
	
    usbMsgPtr = replyBuffer;
    return replyLength;
}


ISR(TWI_vect)
{
	static unsigned char i2c_state;
	unsigned char twi_status;

   // Disable Global Interrupt
   cli();

   // Get TWI Status Register, mask the prescaler bits (TWPS1,TWPS0)
   twi_status = TWSR & 0xF8;     

   switch(twi_status) {
     case TW_SR_SLA_ACK:     	 // 0x60: SLA+W received, ACK returned

      	  i2c_state=0;           // Start I2C State for Register Address required	 

       	  TWCR |= (1<<TWINT);    // Clear TWINT Flag
          break;

     case TW_SR_DATA_ACK:    	 // 0x80: data received, ACK returned

			switch(i2c_state){				
     			case 0:									//====[REG_ID]=============================
             			REG_ID = TWDR;  
						REG_VH = 0;
						REG_VL = 0;						    
	 		 			i2c_state = 1;
						break;
     			case 1:									//====[REG_VALUE 8줸]=============================
             			REG_VH = TWDR;      
	 		 			i2c_state = 2;
						break;
     			case 2:									//====[REG_VALUE C8줸]=============================
             			REG_VL = TWDR;     
	 		 			i2c_state = 3;

						switch(REG_ID){				
			     			case 0:	
									break;
			     			case 1:									//====[n1]=======
									VAL_ch1 = (REG_VH << 8 );
									VAL_ch1 = (VAL_ch1 | REG_VL);
									break;
			     			case 2:									//====[n2]=======
									VAL_ch2 = (REG_VH << 8 );
									VAL_ch2 = (VAL_ch2 | REG_VL);
									break;
			     			case 3:									//====[n3]=======	
									VAL_ch3 = (REG_VH << 8 );
									VAL_ch3 = (VAL_ch3 | REG_VL);
									break;
			     			case 4:									//====[n4]=======
									VAL_ch4 = (REG_VH << 8 );
									VAL_ch4 = (VAL_ch4 | REG_VL);
									break;
			     			case 5:									//====[n5]=======
									VAL_ch5 = (REG_VH << 8 );
									VAL_ch5 = (VAL_ch5 | REG_VL);							
									break;
			     			case 6:									//====[n6]=======
									VAL_ch6 = (REG_VH << 8 );
									VAL_ch6 = (VAL_ch6 | REG_VL);							
									break;
			     			case 7:	
									break;
			     			case 8:	
									break;
			     			case 9:								//====[׻]=======
									VAL_angle = (REG_VH << 8 );
									VAL_angle = (VAL_angle | REG_VL);
									break;

			     			case 10:							//====[t]=======
									VAL_gyro = (REG_VH << 8 );
									VAL_gyro = (VAL_gyro | REG_VL);
									break;
			     			case 20:							//====[buttons]=======
									VAL_buttons = (REG_VH << 8 );
									VAL_buttons = (VAL_buttons | REG_VL);
									break;									
						}

						break;
 			}
 

          TWCR |= (1<<TWINT);    // Clear TWINT Flag
          break;

     case TW_SR_STOP:        	 // 0xA0: stop or repeated start condition received while selected

    		TWCR |= (1<<TWINT);    // Clear TWINT Flag

			if (i2c_state == 3) {
				i2c_state = 0;
			}	   
 
        	break;

     case TW_ST_DATA_NACK:    // 0xC0: data transmitted, NACK received
     case TW_ST_LAST_DATA:    // 0xC8: last data byte transmitted, ACK received
     case TW_BUS_ERROR:       // 0x00: illegal start or stop condition
     default:
		  	TWCR |= (1<<TWINT);    // Clear TWINT Flag
			i2c_state = 0;         // Back to the Begining State
   }

   // Enable Global Interrupt
   sei();
}



/**
 * Main-function. Initializes the hardware and starts the main loop of the
 * application.
 * \return An integer. Whatever... :-)
 */
int main(void) {
    uchar i, j;
//    odDebugInit();
	
    DDRD = ~0;                  /* output SE0 for USB reset */
    PORTD = 0x00;               /* no pullups on USB pins */

    j = 0;
    while (--j) {               /* USB Reset by device only required on
                                   Watchdog Reset */
        i = 0;
        while (--i);            /* delay >10ms for USB reset */
    }
    DDRD = ~USBMASK;            /* all outputs except USB data */ 
    TCCR0 = 5;                  /* set prescaler to 1/1024 */
	
 	//====[Initial I2C Slave]===================================================================================
  	TWAR = (0x02 << 1); 	   //====[]wqa} 0x02]=======
  	TWDR = 0x00;               //====[ks ]===============
  	TWBR = 0x30;

	//====[Start Slave Listening: Clear TWINT Flag, Enable ACK, Enable TWI, TWI Interrupt Enable]=========
  	TWCR = (1<<TWINT) | (1<<TWEA) | (1<<TWEN) | (1<<TWIE);

    usbInit();
    sei();

    for (i = 0; i < 10; i++) {  /* wait one second to prevent strange
                                   effects when the USB-bus isn't
                                   initialized (e. g. when the host system
                                   is on standby. */
        _delay_ms(100);
    }

    angle = 0;

    while (1) {                 /* main event loop */
	        usbPoll();
			_delay_ms(25);				
	}
    return 0;
}
