/**
 * \file main.c
 * \brief Firmware for the USB-Servo
 * \author Ronald Schaten
 * \version $Id: main.c,v 1.1 2006/10/28 12:40:42 rschaten Exp $
 *
 * License: See documentation.
 */

#define F_CPU 12000000UL
 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <compat/twi.h>

#define MAX_TRIES 50

#define I2C_START 0
#define I2C_DATA 1
#define I2C_DATA_ACK 2
#define I2C_STOP 3
#define ACK 1
#define NACK 0

#include "usbdrv.h"
#include "oddebug.h"
#include "usbservo.h"


extern volatile unsigned char DISPLAY1_VH;			//====[Set Value High]=========================
extern volatile unsigned char DISPLAY1_VL;			//====[Set Value Low]==========================
extern volatile unsigned char DISPLAY2_VH;			//====[Set Value High]=========================
extern volatile unsigned char DISPLAY2_VL;			//====[Set Value Low]==========================

volatile unsigned char DISPLAY1_VH = 0;  
volatile unsigned char DISPLAY1_VL = 0;   
volatile unsigned char DISPLAY2_VH = 0;  
volatile unsigned char DISPLAY2_VL = 0;   

extern volatile unsigned int ch1,ch2,ch3,ch4,ch5,ch6,ch7,ch8;

volatile unsigned int ch1 = 0;
volatile unsigned int ch2 = 0;
volatile unsigned int ch3 = 0;
volatile unsigned int ch4 = 0;
volatile unsigned int ch5 = 0;
volatile unsigned int ch6 = 0;
volatile unsigned int ch7 = 0;
volatile unsigned int ch8 = 0;


/** Global variable, contains the angle of the servo. */
static uint8_t angle;

/**
 * USB-Setup-Handler. Handles setup-calls that are received from the
 * USB-Interface.
 * \param data Eight bytes of data.
 * \return The number of returned bytes (in replyBuffer[]).
 */
uchar usbFunctionSetup(uchar data[8]) {
    static uchar replyBuffer[8];
    uchar replyLength;

    replyBuffer[0] = msgOK;
    switch (data[1]) {
    case CMD_GETADC:             /* get ADC Value */
	
		//====[׻]=========================
        replyBuffer[0] = DISPLAY1_VL;
        replyBuffer[1] = DISPLAY1_VH;
		
		//====[]=========================
        replyBuffer[2] = DISPLAY2_VL;
        replyBuffer[3] = DISPLAY2_VH;	
		
		//====[ch1 (½ roll)]================
        replyBuffer[4] = ch1;
        replyBuffer[5] = ch1 >> 8;
		
		//====[ch4 (o)]=====================
        replyBuffer[6] = ch4;
        replyBuffer[7] = ch4 >> 8;		
		
        replyLength = 8;
        break;	
    case CMD_ECHO:             /* echo */
        replyBuffer[0] = data[2];
        replyBuffer[1] = data[3];
        replyLength = 2;
        break;
    case CMD_GET:              /* read status */
        replyBuffer[0] = angle;
        replyLength = 1;
        break;
    case CMD_SET:              /* set status */
        angle = data[2];
        replyLength = 0;
        break;
    default:                   /* WTF? */
        replyBuffer[0] = msgErr;
        replyLength = 1;
        break;
    }
    usbMsgPtr = replyBuffer;
    return replyLength;
}


ISR(TWI_vect)
{
	static unsigned char i2c_state;
	unsigned char twi_status;

   // Disable Global Interrupt
   cli();

   // Get TWI Status Register, mask the prescaler bits (TWPS1,TWPS0)
   twi_status = TWSR & 0xF8;     

   switch(twi_status) {
     case TW_SR_SLA_ACK:     	 // 0x60: SLA+W received, ACK returned

      	  i2c_state=0;           // Start I2C State for Register Address required	 

       	  TWCR |= (1<<TWINT);    // Clear TWINT Flag
          break;

     case TW_SR_DATA_ACK:    	 // 0x80: data received, ACK returned

			switch(i2c_state){
				//====[׻]=============================
     			case 0:
             			DISPLAY1_VH = TWDR;      
	 		 			i2c_state = 1;
						break;
     			case 1:
             			DISPLAY1_VL = TWDR;      
	 		 			i2c_state = 2;
						break;
				//====[]=============================
     			case 2:
             			DISPLAY2_VH = TWDR;     
	 		 			i2c_state = 3;
						break;
     			case 3:
             			DISPLAY2_VL = TWDR;      
	 		 			i2c_state = 4;
						break;
				//====[n1]==============================
     			case 4:
						ch1 = 0;
             			ch1 = TWDR << 8;     
	 		 			i2c_state = 5;
						break;
     			case 5:
             			ch1 = (ch1 | TWDR);     
	 		 			i2c_state = 6;
						break;
				//====[n2]==============================
     			case 6:
 						ch2 = 0;
             			ch2 = TWDR << 8;     
	 		 			i2c_state = 7;
						break;
     			case 7:
             			ch2 = (ch2 | TWDR); 
	 		 			i2c_state = 8;
						break;
				//====[n3]==============================
     			case 8:
 						ch3 = 0;
             			ch3 = TWDR << 8;     
	 		 			i2c_state = 9;
						break;
     			case 9:
             			ch3 = (ch3 | TWDR); 
	 		 			i2c_state = 10;
						break;
				//====[n4]==============================
     			case 10:
						ch4 = 0;
             			ch4 = TWDR << 8;        
	 		 			i2c_state = 11;
						break;
     			case 11:
             			ch4 = (ch4 | TWDR); 
	 		 			i2c_state = 12;
						break;
 			}
 

          TWCR |= (1<<TWINT);    // Clear TWINT Flag
          break;

     case TW_SR_STOP:        	 // 0xA0: stop or repeated start condition received while selected

    		TWCR |= (1<<TWINT);    // Clear TWINT Flag

			if (i2c_state == 12) {
				i2c_state = 0;
			}	   
 
        	break;

     case TW_ST_DATA_NACK:    // 0xC0: data transmitted, NACK received
     case TW_ST_LAST_DATA:    // 0xC8: last data byte transmitted, ACK received
     case TW_BUS_ERROR:       // 0x00: illegal start or stop condition
     default:
		  	TWCR |= (1<<TWINT);    // Clear TWINT Flag
			i2c_state = 0;         // Back to the Begining State
   }

   // Enable Global Interrupt
   sei();
}



/**
 * Main-function. Initializes the hardware and starts the main loop of the
 * application.
 * \return An integer. Whatever... :-)
 */
int main(void) {
    uchar i, j;
//    odDebugInit();
	
    DDRD = ~0;                  /* output SE0 for USB reset */
    PORTD = 0x00;               /* no pullups on USB pins */

    j = 0;
    while (--j) {               /* USB Reset by device only required on
                                   Watchdog Reset */
        i = 0;
        while (--i);            /* delay >10ms for USB reset */
    }
    DDRD = ~USBMASK;            /* all outputs except USB data */
    TCCR0 = 5;                  /* set prescaler to 1/1024 */
	
 	//====[Initial I2C Slave]===================================================================================
  	TWAR = 0x04; 			   //====[]wqa} 2]=======
  	TWDR = 0x00;               //====[ks ]===============
  	TWBR = 0x30;

	//====[Start Slave Listening: Clear TWINT Flag, Enable ACK, Enable TWI, TWI Interrupt Enable]=========
  	TWCR = (1<<TWINT) | (1<<TWEA) | (1<<TWEN) | (1<<TWIE);

    usbInit();
    sei();

    for (i = 0; i < 10; i++) {  /* wait one second to prevent strange
                                   effects when the USB-bus isn't
                                   initialized (e. g. when the host system
                                   is on standby. */
        _delay_ms(100);
    }

    angle = 0;

    while (1) {                 /* main event loop */
	        usbPoll();
			_delay_ms(20);			
	}
    return 0;
}
